<?php

namespace Drupal\watchdog_mailer\Logger;

use Drupal\Component\Render\FormattableMarkup;
use Drupal\Component\Utility\Html;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Logger\RfcLoggerTrait;
use Drupal\Core\Logger\RfcLogLevel;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Psr\Log\LoggerInterface;

/**
 * {@inheritdoc}
 */
class WatchdogMailer implements LoggerInterface {
  use RfcLoggerTrait;
  use StringTranslationTrait;

  /**
   * The config factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $config;

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * Constructs a new WatchdogMailer.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   A config factory for retrieving required config objects.
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   The language manager.
   */
  public function __construct(ConfigFactoryInterface $config_factory, LanguageManagerInterface $language_manager) {
    $this->config = $config_factory->get('watchdog_mailer.settings');
    $this->languageManager = $language_manager;
  }

  /**
   * {@inheritdoc}
   */
  public function log($level, $message, array $context = []) {
    $enabled = $this->config->get('watchdog_mailer_enable');
    $channelsEnable = $this->config->get('channels_enable');
    $channelsNegate = $this->config->get('channels_negate');
    $channels = explode("\r\n", mb_strtolower($this->config->get('channels_list')));

    if ($channelsEnable && !empty($channels)) {
      if (
        !$channelsNegate && !in_array($context['channel'], $channels)
        ||
        $channelsNegate && in_array($context['channel'], $channels)
      ) {
        $enabled = 0;
      }
    }

    if ($enabled) {
      try {
        if ($level <= RfcLogLevel::WARNING || ($this->config->get('php_notices') && $context['channel'] == 'php')) {
          $body = new FormattableMarkup($message, $context);
          $body = Html::decodeEntities(strip_tags($body->__toString()));
          $params = [
            'subject' => $this->t('Error in the database log: @channel', ['@channel' => $context['channel']]),
            'body' => $body,
          ];
          $langcode = $this->languageManager->getCurrentLanguage()->getId();
          // @codingStandardsIgnoreLine
          if (!\Drupal::service('plugin.manager.mail')->mail(
            'watchdog_mailer',
            'watchdog_mailer',
            $this->config->get('mail_address'),
            $langcode,
            $params)
          ) {
            throw new Exception('There was a problem sending the message.');
          }
        }
      }
      catch (Exception $e) {
        // There can be no watchdog_exception to avoid a possible endless cycle.
        $this->notice($e->getMessage());
      }
    }
  }

}
